#include "component.h"
#include <iostream>
#include "parameters.h"
#include "uniform.h"
#include "normal.h"
#include "circular_average.h"
#include "math.h"
#include "clique.h"

float degree_squares = 0;
float central_core_ratio = 0;

Component::Component(int m_shellIndex, Component* m_parentComponent) {
	shellIndex = m_shellIndex;
	size = 0;
	shellCardinal = 0;
	parentComponent = m_parentComponent;

	rho = -1.0;
	phi = 0.0;

	xCoord = 0.0;
	yCoord = 0.0;
	u = parameters.getU();
}

void Component::placeInCircularSector(float ratio, float alfa, int n, int total, float random, float& rho, float& phi) {
	float pos = ((float)n / total) + random;
	if (pos > 1.0)
		pos = pos - 1.0;

	float divs;
	if (alfa != 2.0 * pi)
		divs = pos * (2.0 * ratio + ratio * alfa);
	else
		divs = ratio + pos * ratio * alfa;

	rho = 0.0;
	phi = 0.0;
	if (divs < ratio) {
		rho = divs;
		phi = 0.0;
	} else {
		if ((divs >= ratio)&&(divs < ratio + ratio * alfa)) {
			rho = ratio;
			phi = (divs - ratio) / ratio;
		} else {
			rho = ratio - (divs - ratio - (ratio * alfa));
			phi = alfa;
		}
	}
}

void Component::neighboursComponentPropagation(Vertex* v, Component* component, std::list<Vertex*>* comp_cluster) {
	std::vector<Vertex*>* neighb = v->getNeighbours();
	std::vector<Vertex*>::iterator neighb_it;
	for (neighb_it=neighb->begin();neighb_it!=neighb->end();neighb_it++) {
		if (((*neighb_it)->getShellIndex() > shellIndex)&&((*neighb_it)->getMark()!=shellIndex+1)) {
			//Assigning component to vertex
			(*neighb_it)->setComponent(component);
			//Introducing vertex in component
			comp_cluster->push_back(*neighb_it);
			//Total size of this component
			component->size++;
			component->all_vertices.push_back(*neighb_it);
			(*neighb_it)->setMark(shellIndex+1);
			neighboursComponentPropagation(*neighb_it, component, comp_cluster);
		}
	}
}

void Component::neighboursClusterPropagation(Vertex* v, std::list<Vertex*>* cluster) {
	std::vector<Vertex*>* neighb = v->getNeighbours();
	std::vector<Vertex*>::iterator neighb_it;
	for (neighb_it=neighb->begin();neighb_it!=neighb->end();neighb_it++) {
		if (((*neighb_it)->getCluster()==NULL)&&((*neighb_it)->getShellIndex()==shellIndex)) {
			//Assigning cluster to vertex
			(*neighb_it)->setCluster(cluster);
			//Introducing vertex in cluster
			cluster->push_back(*neighb_it);
			neighboursClusterPropagation(*neighb_it, cluster);
		}
	}
}

void Component::computeComponents(float &central_core_parent, float maxShellIndex, float minShellIndex, float maxDegree) {

	std::list<Vertex*>* vertices = clusters.front();

	std::list<Vertex*>::iterator vertex_it;
	Vertex* v;
	std::list<Vertex*>* cluster = new std::list<Vertex*>();
	for (vertex_it=vertices->begin(); vertex_it!=vertices->end(); vertex_it++) {
		v = *vertex_it;
		//Vertexes which should be in an internal core are marked with a number equal to this core + 1		

		//If this vertex must be in an internal core
		if (v->getMark()!=shellIndex+1) {
			if (v->getShellIndex() > shellIndex) {
				Component* component = new Component(shellIndex+1, this);
				components.push_back(component);

				//Cluster for this new component
				std::list<Vertex*>* comp_cluster = new std::list<Vertex*>();
				component->clusters.push_back(comp_cluster);
				//Assigning component to vertex
				v->setComponent(component);
				//Introducing vertex in component
				comp_cluster->push_back(v);
				//Total size of this component
				component->size++;
				component->all_vertices.push_back(v);
				v->setMark(shellIndex+1);

				neighboursComponentPropagation(v, component, comp_cluster);
			}					
			else {
				shellCardinal++;
				//If this vertex belongs to a cluster in this shell
				cluster->push_back(v);
			}
		}
	}
	clusters.push_back(cluster);	
	//Deleting cluster that contained all this core
	clusters.pop_front();

	//Clusters computation
	for (vertex_it=clusters.front()->begin(); vertex_it!=clusters.front()->end(); vertex_it++) {
		v = *vertex_it;

		if (v->getCluster()==NULL) {
			std::list<Vertex*>* n_cluster = new std::list<Vertex*>();
			//Assigning cluster to vertex
			v->setCluster(n_cluster);
			//Introducing vertex in cluster
			n_cluster->push_back(v);
			neighboursClusterPropagation(v, n_cluster);
			
			clusters.push_back(n_cluster);
		}
	
	}
	//Deleting cluster that contained all the clusters from this shell
	clusters.pop_front();

	std::list<std::list<Vertex*>*> new_clusters;
	std::list<std::list<Vertex*>*>::iterator new_clusters_it;
	Uniform uniform(0.00, 1.00);
	for (new_clusters_it=clusters.begin(); new_clusters_it!=clusters.end(); new_clusters_it++) {
		if (uniform.getValue()>0.5) {
			new_clusters.push_back(*new_clusters_it);
		}
		else {
			new_clusters.push_front(*new_clusters_it);
		}
	}
	clusters = new_clusters;

	std::list<Component*>::iterator components_it;

/*	else {
		float c2 = 0.0;
		float c3;
		for (components_it=components.begin();components_it!=components.end();components_it++) {
			(*components_it)->computeComponents(c3, maxShellIndex, minShellIndex, maxDegree);
			c2 = c3;
		}
		central_core_ratio = c2;
		central_core_parent = c2;
	}
*/	for (components_it=components.begin();components_it!=components.end();components_it++) {
		(*components_it)->computeComponents(central_core_parent, maxShellIndex, minShellIndex, maxDegree);
	}
}

void Component::computeComponents(std::list<Vertex*>* vertices, float maxShellIndex, float minShellIndex, float maxDegree) {
	clusters.push_front(vertices);
	all_vertices = *vertices;
	size += vertices->size();
	float c;
	this->computeComponents(c, maxShellIndex, minShellIndex, maxDegree);
}

void Component::printComponent() {
	if (clusters.size()!=0) {
		for (int i=0;i<shellIndex;i++) std::cout << " ";			
		//std::cout << "Component of core " << core << std::endl;
	}

	std::list<Component*>::iterator components_it;
	std::list<std::list<Vertex*>*>::iterator clusters_it;
	std::list<Vertex*>::iterator cluster_it;
	for (clusters_it=clusters.begin();clusters_it!=clusters.end();clusters_it++) {
		for (int i=0;i<shellIndex;i++) std::cout << " ";			
		for (cluster_it=(*clusters_it)->begin();cluster_it!=(*clusters_it)->end();cluster_it++) {
			std::cout << (*cluster_it)->getNumber() << " ";
		}
		std::cout << std::endl;
	}
	for (components_it=components.begin();components_it!=components.end();components_it++) {
		(*components_it)->printComponent();
	}
}

//Sumatory of previous components cardinals
int Component::getBrotherComponentsCardinal() {
	int sumatory = 0;
	std::list<Component*> brothers = getParentComponent()->getComponents();
	std::list<Component*>::iterator it;
	for (it=brothers.begin();it!=brothers.end();it++)
		sumatory += (*it)->getSize();
	return sumatory;
}
	
//Sumatory of previous components cardinals
int Component::getPreviousComponentsCardinal() {
	int sumatory = 0;
	std::list<Component*> brothers = getParentComponent()->getComponents();
	std::list<Component*>::iterator it;
	for (it=brothers.begin();*it!=this;it++)
		sumatory += (*it)->getSize();
	sumatory += (*it)->getSize() / 2;
	return sumatory;
}

void Component::buildClustersMap(std::map<int, std::list<std::list<Vertex*>* > >& clustersMap) {
	std::list<std::list<Vertex*>* >::iterator it_clusters;
	for (it_clusters=clusters.begin();it_clusters!=clusters.end();it_clusters++) {
		clustersMap[shellIndex].push_back(*it_clusters);
	}

	std::list<Component*>::iterator it_comp;
	for (it_comp=components.begin();it_comp!=components.end();it_comp++) {
		(*it_comp)->buildClustersMap(clustersMap);
	}
}

